
import import_declare_test

from splunktaucclib.rest_handler.endpoint import (
    field,
    validator,
    RestModel,
    SingleModel,
)
from splunktaucclib.rest_handler import admin_external, util
from splunktaucclib.rest_handler.admin_external import AdminExternalHandler
import logging

util.remove_http_proxy_env_vars()


fields = [
    field.RestField(
        'alias',
        required=False,
        encrypted=False,
        default=None,
        validator=validator.Pattern(
            regex=r"""^.+$""", 
        )
    ), 
    field.RestField(
        'description',
        required=False,
        encrypted=False,
        default=None,
        validator=validator.AllOf(
            validator.String(
                max_len=100, 
                min_len=1, 
            ), 
            validator.Pattern(
                regex=r"""^.*$""", 
            )
        )
    ), 
    field.RestField(
        'default_priority',
        required=True,
        encrypted=False,
        default='medium',
        validator=None
    ), 
    field.RestField(
        'pagination_mode',
        required=True,
        encrypted=False,
        default='local',
        validator=None
    ), 
    field.RestField(
        'pagination_size',
        required=True,
        encrypted=False,
        default='10000',
        validator=validator.Pattern(
            regex=r"""^\d*$""", 
        )
    ), 
    field.RestField(
        'ui_default_timerange',
        required=True,
        encrypted=False,
        default='24h',
        validator=None
    ), 
    field.RestField(
        'ui_min_object_width',
        required=True,
        encrypted=False,
        default='300',
        validator=validator.Pattern(
            regex=r"""^[0-9]*$""", 
        )
    ), 
    field.RestField(
        'ui_expand_metrics',
        required=True,
        encrypted=False,
        default='0',
        validator=None
    ), 
    field.RestField(
        'ui_home_tabs_order',
        required=True,
        encrypted=False,
        default='dsm,cim,flx,dhm,mhm,wlk,flip,audit,alerts',
        validator=None
    ), 
    field.RestField(
        'outliers_set_state',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'data_sampling_set_state',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'data_sampling_obfuscation',
        required=True,
        encrypted=False,
        default='0',
        validator=None
    ), 
    field.RestField(
        'adaptive_delay',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'mloutliers',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'sampling',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'splk_feeds_auto_disablement_period',
        required=True,
        encrypted=False,
        default='60d',
        validator=validator.Pattern(
            regex=r"""^[0-9]*d$""", 
        )
    ), 
    field.RestField(
        'indexed_constraint',
        required=False,
        encrypted=False,
        default='',
        validator=None
    ), 
    field.RestField(
        'cmdb_lookup',
        required=True,
        encrypted=False,
        default='1',
        validator=None
    ), 
    field.RestField(
        'splk_dsm_tabulator_groupby',
        required=True,
        encrypted=False,
        default='data_index',
        validator=None
    ), 
    field.RestField(
        'splk_dhm_tabulator_groupby',
        required=True,
        encrypted=False,
        default='tenant_id',
        validator=None
    ), 
    field.RestField(
        'splk_mhm_tabulator_groupby',
        required=True,
        encrypted=False,
        default='tenant_id',
        validator=None
    ), 
    field.RestField(
        'splk_flx_tabulator_groupby',
        required=True,
        encrypted=False,
        default='group',
        validator=None
    ), 
    field.RestField(
        'splk_wlk_tabulator_groupby',
        required=True,
        encrypted=False,
        default='overgroup',
        validator=None
    ), 
    field.RestField(
        'splk_cim_tabulator_groupby',
        required=True,
        encrypted=False,
        default='cim_datamodel_name',
        validator=None
    )
]
model = RestModel(fields, name=None)


endpoint = SingleModel(
    'trackme_vtenants',
    model,
    config_name='vtenants'
)


if __name__ == '__main__':
    logging.getLogger().addHandler(logging.NullHandler())
    admin_external.handle(
        endpoint,
        handler=AdminExternalHandler,
    )
