#!/usr/bin/env python
# coding=utf-8

__author__ = "TrackMe Limited"
__copyright__ = "Copyright 2023-2024, TrackMe Limited, U.K."
__credits__ = "TrackMe Limited, U.K."
__license__ = "TrackMe Limited, all rights reserved"
__version__ = "0.1.0"
__maintainer__ = "TrackMe Limited, U.K."
__email__ = "support@trackme-solutions.com"
__status__ = "PRODUCTION"

# default vtenant account configuration settings
vtenant_account_default = {
    "ui_default_timerange": "24h",
    "ui_min_object_width": 300,
    "data_sampling_set_state": 1,
    "outliers_set_state": 1,
    "data_sampling_obfuscation": 0,
    "ui_expand_metrics": 0,
    "ui_home_tabs_order": "dsm,flx,dhm,mhm,wlk,cim,flip,audit,alerts",
    "adaptive_delay": 1,
    "mloutliers": 1,
    "sampling": 1,
    "adaptive_delay": 1,
    "indexed_constraint": "",
    "splk_feeds_auto_disablement_period": "90d",
    "cmdb_lookup": 1,
    "default_priority": "medium",
    "pagination_mode": "local",
    "pagination_size": 10000,
    "splk_dsm_tabulator_groupby": "data_index",
    "splk_dhm_tabulator_groupby": "tenant_id",
    "splk_mhm_tabulator_groupby": "tenant_id",
    "splk_flx_tabulator_groupby": "group",
    "splk_wlk_tabulator_groupby": "overgroup",
    "splk_cim_tabulator_groupby": "cim_datamdel_name",
}

# main and shared collections
collections_list_main = [
    "trackme_virtual_tenants",
    "trackme_virtual_tenants_entities_summary",
    "trackme_maintenance_mode",
    "trackme_maintenance_kdb",
    "trackme_backup_archives_info",
]

# dict of collections
collections_dict = dict(
    [
        (
            "trackme_dsm",
            "_key, ctime, mtime, search_mode, tenant_id, object_category, alias, data_index, data_last_lag_seen, data_last_ingestion_lag_seen, data_eventcount, data_last_lag_seen_idx, data_first_time_seen, data_last_time_seen, data_last_ingest, data_last_time_seen_idx, data_max_lag_allowed, data_max_delay_allowed, future_tolerance, data_lag_alert_kpis, monitored_state, object, data_sourcetype, data_monitoring_wdays, isUnderMonitoringDays, data_monitoring_hours_ranges, isUnderMonitoringHours, data_override_lagging_class, allow_adaptive_delay, object_state, tracker_runtime, object_previous_state, data_previous_tracker_runtime, dcount_host, min_dcount_host, min_dcount_field, min_dcount_threshold, avg_dcount_host_5m, latest_dcount_host_5m, perc95_dcount_host_5m, stdev_dcount_host_5m, global_dcount_host, isAnomaly, data_sample_lastrun, tags, tags_manual, latest_flip_state, latest_flip_time, priority, sla_class, status_message, anomaly_reason, outliers_readiness, tenant_parent",
        ),
        (
            "trackme_dsm_elastic_shared",
            "_key, object, alias, search_constraint, search_mode, elastic_index, elastic_sourcetype, earliest, latest",
        ),
        (
            "trackme_dsm_elastic_dedicated",
            "_key, object, alias, search_constraint, search_mode, elastic_index, elastic_sourcetype, elastic_wrapper, elastic_report",
        ),
        ("trackme_dsm_hybrid_trackers", "_key, tracker_name, knowledge_objects"),
        (
            "trackme_dsm_allowlist",
            "_key, object_category, object, is_rex, action, comment, mtime",
        ),
        (
            "trackme_dsm_data_sampling",
            "_key, object, raw_sample, events_count, min_time_btw_iterations_seconds, pct_min_major_inclusive_model_match, pct_max_exclusive_model_match, pct_max_exclusive_model_match, relative_time_window_seconds, data_sample_mtime, data_sample_last_entity_epoch_processed, data_sample_model_matched_summary, data_sample_feature, data_sample_iteration, data_sample_anomaly_reason, data_sample_status_colour, data_sample_anomaly_detected, data_sample_status_message, multiformat_detected, current_detected_format, current_detected_format_id, current_detected_format_dcount, current_detected_major_format, previous_detected_format, previous_detected_format_id, previous_detected_format_dcount, previous_detected_major_format",
        ),
        (
            "trackme_dsm_data_sampling_custom_models",
            "_key, model_id, model_name, model_regex, model_type, sourcetype_scope, mtime",
        ),
        ("trackme_dsm_knowledge", "_key, object, doc_link, doc_note"),
        (
            "trackme_dsm_outliers_entity_rules",
            "_key, object_category, object, mtime, is_disabled, entities_outliers, last_exec, confidence, confidence_reason",
        ),
        (
            "trackme_dsm_outliers_entity_data",
            "_key, object_category, object, mtime, isOutlier, isOutlierReason, models_in_anomaly, models_summary",
        ),
        (
            "trackme_dsm_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_dsm_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_dsm_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_dsm_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_dsm_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_dsm_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        (
            "trackme_cim",
            "_key, ctime, mtime, tenant_id, object, object_category, alias, monitored_state, cim_tracking_rules, cim_tracking_results, object_state, account, monitoring_wdays, isUnderMonitoringDays, monitoring_hours_ranges, isUnderMonitoringHours, tracker_name, tracker_runtime, tracker_last_duration, tags, tags_manual, latest_flip_state, latest_flip_time, priority, sla_class, anomaly_reason, outliers_readiness, tenant_parent",
        ),
        (
            "trackme_cim_simulation",
            "_key, tenant_id, object, object_category, alias, monitored_state, cim_tracking_rules, cim_tracking_results, object_state, account, monitoring_wdays, isUnderMonitoringDays, monitoring_hours_ranges, isUnderMonitoringHours, tracker_name, tracker_runtime, tracker_last_duration, tags, tags_manual, latest_flip_state, latest_flip_time, priority, anomaly_reason",
        ),
        (
            "trackme_cim_outliers_entity_rules",
            "_key, object_category, object, entity, cim_field, mtime, is_disabled, entities_outliers, last_exec, confidence, confidence_reason",
        ),
        (
            "trackme_cim_outliers_entity_data",
            "_key, object_category, object, entity, cim_field, mtime, isOutlier, isOutlierReason, models_in_anomaly, models_summary",
        ),
        (
            "trackme_cim_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_cim_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_cim_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_cim_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_cim_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_cim_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        (
            "trackme_flx",
            "_key, ctime, mtime, tenant_id, group, object, object_category, object_state, object_description, alias, monitored_state, account, tracker_name, tracker_runtime, status, status_description, metrics, outliers_metrics, extra_attributes, monitoring_wdays, isUnderMonitoringDays, monitoring_hours_ranges, isUnderMonitoringHours, tags, tags_manual, latest_flip_state, latest_flip_time, priority, sla_class, status_message, anomaly_reason, outliers_readiness, tenant_parent, max_sec_inactive",
        ),
        (
            "trackme_flx_hybrid_trackers",
            "_key, tracker_id, tracker_name, knowledge_objects",
        ),
        (
            "trackme_flx_outliers_entity_rules",
            "_key, object_category, object, mtime, is_disabled, entities_outliers, last_exec, confidence, confidence_reason",
        ),
        (
            "trackme_flx_outliers_entity_data",
            "_key, object_category, object, mtime, isOutlier, isOutlierReason, models_in_anomaly, models_summary",
        ),
        (
            "trackme_flx_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_flx_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_flx_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_flx_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_flx_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_flx_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        (
            "trackme_wlk",
            "_key, ctime, mtime, tenant_id, overgroup, group, app, user, savedsearch_name, object, object_category, object_state, object_description, alias, monitored_state, account, tracker_name, tracker_runtime, status, status_description, metrics, metrics_extended, outliers_metrics, monitoring_wdays, isUnderMonitoringDays, monitoring_hours_ranges, isUnderMonitoringHours, first_seen, last_seen, sec_since_lastexec, skipped_pct, skipped_pct_last_60m, skipped_pct_last_4h, skipped_pct_last_24h, count_errors, count_errors_last_60m, count_errors_last_4h, count_errors_last_24h, tags, tags_manual, latest_flip_state, latest_flip_time, priority, sla_class, tags, status_message, anomaly_reason, outliers_readiness, tenant_parent",
        ),
        (
            "trackme_wlk_hybrid_trackers",
            "_key, tracker_id, tracker_name, knowledge_objects",
        ),
        (
            "trackme_wlk_outliers_entity_rules",
            "_key, object_category, object, mtime, is_disabled, entities_outliers, last_exec, confidence, confidence_reason",
        ),
        (
            "trackme_wlk_outliers_entity_data",
            "_key, object_category, object, mtime, isOutlier, isOutlierReason, models_in_anomaly, models_summary",
        ),
        (
            "trackme_wlk_versioning",
            "_key, mtime, object, version_dict, description, current_version_id, cron_exec_sequence_sec",
        ),
        ("trackme_wlk_orphan_status", "_key, mtime, object, user, app, orphan"),
        ("trackme_wlk_apps_enablement", "_key, mtime, app, enabled"),
        (
            "trackme_wlk_last_seen_activity",
            "_key, account, object, last_seen_scheduler, last_seen_introspection, last_seen_notable, last_seen_splunkcloud_svc",
        ),
        (
            "trackme_wlk_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_wlk_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_wlk_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_wlk_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_wlk_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_wlk_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        (
            "trackme_dhm",
            "_key, ctime, mtime, search_mode, tenant_id, object_category, object, alias, data_index, data_sourcetype, data_last_lag_seen, data_last_ingestion_lag_seen, data_eventcount, data_first_time_seen, data_last_time_seen, data_last_ingest, data_max_lag_allowed, data_max_delay_allowed, future_tolerance, data_lag_alert_kpis, monitored_state, data_monitoring_wdays, isUnderMonitoringDays, data_monitoring_hours_ranges, isUnderMonitoringHours, data_override_lagging_class, allow_adaptive_delay, object_state, tracker_runtime, object_previous_state, data_previous_tracker_runtime, splk_dhm_st_summary, splk_dhm_st_summary_full, splk_dhm_st_summary_compact, splk_dhm_st_summary_minimal, splk_dhm_alerting_policy, host_idx_blocklists, host_st_blocklists, tags, tags_manual, latest_flip_state, latest_flip_time, priority, status_message, anomaly_reason, outliers_readiness, tenant_parent",
        ),
        (
            "trackme_dhm_allowlist",
            "_key, object_category, object, is_rex, action, comment, mtime",
        ),
        (
            "trackme_dhm_outliers_entity_rules",
            "_key, object_category, object, mtime, is_disabled, entities_outliers, last_exec, confidence, confidence_reason",
        ),
        (
            "trackme_dhm_outliers_entity_data",
            "_key, object_category, object, mtime, isOutlier, isOutlierReason, models_in_anomaly, models_summary",
        ),
        (
            "trackme_dhm_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_dhm_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_dhm_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_dhm_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_dhm_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_dhm_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        ("trackme_dhm_hybrid_trackers", "_key, tracker_name, knowledge_objects"),
        (
            "trackme_mhm",
            "_key, ctime, mtime, tenant_id, object_category, object, alias, metric_index, metric_category, metric_details, metric_details_full, metric_details_compact, metric_details_minimal, metric_last_lag_seen, metric_first_time_seen, metric_last_time_seen, metric_max_lag_allowed, monitored_state, metric_override_lagging_class, object_state, tracker_runtime, object_previous_state, metric_previous_tracker_runtime, tags, tags_manual, latest_flip_state, latest_flip_time, priority, status_message, anomaly_reason, tenant_parent",
        ),
        (
            "trackme_mhm_allowlist",
            "_key, object_category, object, is_rex, action, comment, mtime",
        ),
        (
            "trackme_common_lagging_classes",
            "_key, level, name, object, value_lag, value_delay",
        ),
        (
            "trackme_mhm_lagging_classes",
            "_key, metric_category, metric_max_lag_allowed",
        ),
        ("trackme_mhm_hybrid_trackers", "_key, tracker_name, knowledge_objects"),
        (
            "trackme_common_audit_changes",
            "_key, time, action, change_type, object, object_category, object_attrs, user, result, comment",
        ),
        (
            "trackme_mhm_tags",
            "_key, object, tags_auto",
        ),
        (
            "trackme_mhm_tags_policies",
            "_key, tags_policy_id, tags_policy_value, tags_policy_regex, mtime",
        ),
        (
            "trackme_mhm_priority",
            "_key, object, priority, priority_reason, mtime",
        ),
        (
            "trackme_mhm_priority_policies",
            "_key, priority_policy_id, priority_policy_value, priority_policy_regex, mtime",
        ),
        (
            "trackme_mhm_sla",
            "_key, object, sla_class, sla_reason, mtime",
        ),
        (
            "trackme_mhm_sla_policies",
            "_key, sla_policy_id, sla_policy_value, sla_policy_regex, mtime",
        ),
        (
            "trackme_common_alerts_ack",
            "_key, object, object_category, anomaly_reason, ack_source, ack_mtime, ack_expiration, ack_state, ack_type, ack_comment",
        ),
        (
            "trackme_common_logical_group",
            "_key, object_group_name, object_group_min_green_percent, object_group_members, object_group_members_green, object_group_members_red, object_group_mtime",
        ),
        (
            "trackme_common_permanently_deleted_objects",
            "_key, object, object_category, ctime",
        ),
        ("trackme_common_replica_trackers", "_key, tracker_name, knowledge_objects"),
        (
            "trackme_common_smartstatus_alert_action_last_seen_activity",
            "_key, object_category, object, last_seen_activity",
        ),
    ]
)

# DSM
collections_list_dsm = [
    "trackme_dsm",
    "trackme_dsm_elastic_shared",
    "trackme_dsm_elastic_dedicated",
    "trackme_dsm_hybrid_trackers",
    "trackme_dsm_allowlist",
    "trackme_dsm_data_sampling",
    "trackme_dsm_data_sampling_custom_models",
    "trackme_dsm_knowledge",
    "trackme_dsm_outliers_entity_rules",
    "trackme_dsm_outliers_entity_data",
    "trackme_dsm_tags",
    "trackme_dsm_tags_policies",
    "trackme_dsm_priority",
    "trackme_dsm_priority_policies",
    "trackme_dsm_sla",
    "trackme_dsm_sla_policies",
]

persistent_fields_dsm = [
    "alias",
    "allow_adaptive_delay",
    "data_lag_alert_kpis",
    "data_max_delay_allowed",
    "data_max_lag_allowed",
    "data_monitoring_hours_ranges",
    "data_monitoring_wdays",
    "data_override_lagging_class",
    "future_tolerance",
    "min_dcount_field",
    "min_dcount_host",
    "monitored_state",
    "priority",
    "sla_class",
    "tags_manual",
]

# CIM
collections_list_cim = [
    "trackme_cim",
    "trackme_cim_simulation",
    "trackme_cim_outliers_entity_rules",
    "trackme_cim_outliers_entity_data",
    "trackme_cim_tags",
    "trackme_cim_tags_policies",
    "trackme_cim_priority",
    "trackme_cim_priority_policies",
    "trackme_cim_sla",
    "trackme_cim_sla_policies",
]

persistent_fields_cim = [
    "alias",
    "cim_tracking_rules",
    "monitoring_hours_ranges",
    "monitoring_wdays",
    "monitored_state",
    "priority",
    "sla_class",
    "tags_manual",
]

# FLX
collections_list_flx = [
    "trackme_flx",
    "trackme_flx_hybrid_trackers",
    "trackme_flx_outliers_entity_rules",
    "trackme_flx_outliers_entity_data",
    "trackme_flx_tags",
    "trackme_flx_tags_policies",
    "trackme_flx_priority",
    "trackme_flx_priority_policies",
    "trackme_flx_sla",
    "trackme_flx_sla_policies",
]

persistent_fields_flx = [
    "alias",
    "monitoring_hours_ranges",
    "monitoring_wdays",
    "monitored_state",
    "priority",
    "sla_class",
    "tags_manual",
]

# DHM
collections_list_dhm = [
    "trackme_dhm",
    "trackme_dhm_allowlist",
    "trackme_dhm_outliers_entity_rules",
    "trackme_dhm_outliers_entity_data",
    "trackme_dhm_hybrid_trackers",
    "trackme_dhm_tags",
    "trackme_dhm_tags_policies",
    "trackme_dhm_priority",
    "trackme_dhm_priority_policies",
    "trackme_dhm_sla",
    "trackme_dhm_sla_policies",
]

persistent_fields_dhm = [
    "alias",
    "allow_adaptive_delay",
    "data_lag_alert_kpis",
    "data_max_delay_allowed",
    "data_max_lag_allowed",
    "data_monitoring_hours_ranges",
    "data_monitoring_wdays",
    "data_override_lagging_class",
    "future_tolerance",
    "host_idx_blocklists",
    "host_st_blocklists",
    "monitored_state",
    "priority",
    "sla_class",
    "splk_dhm_alerting_policy",
    "splk_dhm_st_summary_compact",
    "splk_dhm_st_summary_full",
    "tags_manual",
]

# MHM
collections_list_mhm = [
    "trackme_mhm",
    "trackme_mhm_allowlist",
    "trackme_mhm_lagging_classes",
    "trackme_mhm_hybrid_trackers",
    "trackme_mhm_tags",
    "trackme_mhm_tags_policies",
    "trackme_mhm_priority",
    "trackme_mhm_priority_policies",
    "trackme_mhm_sla",
    "trackme_mhm_sla_policies",
]

persistent_fields_mhm = [
    "alias",
    "metric_details_compact",
    "metric_details_full",
    "metric_max_lag_allowed",
    "metric_override_lagging_class",
    "monitored_state",
    "priority",
    "sla_class",
    "tags_manual",
]

# WLK
collections_list_wlk = [
    "trackme_wlk",
    "trackme_wlk_hybrid_trackers",
    "trackme_wlk_outliers_entity_rules",
    "trackme_wlk_outliers_entity_data",
    "trackme_wlk_versioning",
    "trackme_wlk_orphan_status",
    "trackme_wlk_apps_enablement",
    "trackme_wlk_last_seen_activity",
    "trackme_wlk_tags",
    "trackme_wlk_tags_policies",
    "trackme_wlk_priority",
    "trackme_wlk_priority_policies",
    "trackme_wlk_sla",
    "trackme_wlk_sla_policies",
]

persistent_fields_wlk = [
    "alias",
    "monitoring_hours_ranges",
    "monitoring_wdays",
    "monitored_state",
    "priority",
    "sla_class",
    "tags_manual",
]

# COMMON
collections_list_common = [
    "trackme_common_lagging_classes",
    "trackme_common_audit_changes",
    "trackme_common_alerts_ack",
    "trackme_common_logical_group",
    "trackme_common_permanently_deleted_objects",
    "trackme_common_replica_trackers",
    "trackme_common_smartstatus_alert_action_last_seen_activity",
]
